'use client';

import '~/styles/phone-input.css';
import { useTheme } from 'next-themes';
import { type KeyboardEvent, useState, useTransition } from 'react';
import PhoneInput from 'react-phone-input-2';
import { type z } from 'zod';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import { ChevronLeft, ChevronRight } from 'lucide-react';

import {
  Form,
  FormControl,
  FormDescription,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '~/components/ui/form';
import EnumChoosingField from '~/components/form-fields/EnumChoosingField';
import { Button } from '~/components/ui/button';
import { Progress } from '~/components/ui/progress';
import ImageInputField from '~/components/form-fields/ImageInputField';
import { Input } from '~/components/ui/input';
import { Textarea } from '~/components/ui/textarea';
import CheckboxField from '~/components/form-fields/CheckboxField';
import { FormError } from '~/components/form-fields/FormError';
import { trackEvent } from '~/lib/analytics';

import { anniType, contattatoType, FormSchema, paradontiteType, siNoType } from '../schemas';
import { dentalFormSubmit } from '../server/form';

import FirstStep from './FirstStep';

type FormValues = z.infer<typeof FormSchema>;

const UserForm = () => {
  const { theme } = useTheme();
  const [isPending, startTransition] = useTransition();
  const [step, setStep] = useState(1);
  const [error, setError] = useState<string | undefined>();

  const form = useForm<FormValues>({
    resolver: zodResolver(FormSchema),
    mode: 'onChange',
    defaultValues: {
      parodontite: undefined,
      anni: undefined,
      dentistaVisitaItalia: undefined,
      panaromicaDentale: undefined,
      panoramica: undefined,
      nome: '',
      email: '',
      telefono: '',
      tuaDomanda: '',
      contattato: undefined,
    },
  });

  const stepsData = {
    2: ['parodontite'] as const,
    3: ['anni'] as const,
    4: ['dentistaVisitaItalia'] as const,
    5: ['panaromicaDentale'] as const,
    6: form.getValues('panaromicaDentale') === 'Si' ? (['panoramica'] as const) : ([] as const),
    7: ['nome'] as const,
    8: ['email'] as const,
    9: ['telefono'] as const,
    10: ['tuaDomanda'] as const,
    11: ['contattato'] as const,
  };

  const nextStep = async () => {
    const fields = stepsData[step as keyof typeof stepsData] ?? [];
    const isValid = await form.trigger(fields);
    if (isValid) {
      trackEvent('Form_Step_Complete', {
        step_number: step,
        form_name: 'dental_questionnaire',
      });
      if (step === 5) {
        if (form.getValues('panaromicaDentale') === 'Si') {
          setStep(6);
        } else {
          setStep(7);
        }
      } else if (step === 6) {
        if (form.getValues('panaromicaDentale') === 'Si' && form.getValues('panoramica') == null) {
          form.setError('panoramica', {
            type: 'required',
            message: 'Per favore aggiungi la foto panoramica.',
          });
        } else {
          setStep(7);
        }
      } else {
        setStep(step + 1);
      }
    }
  };

  const prevStep = () => {
    if (step === 7) {
      if (form.getValues('panaromicaDentale') === 'Si') {
        setStep(6);
      } else {
        setStep(5);
      }
    } else if (step === 6) {
      setStep(5);
    } else if (step > 1) {
      setStep(step - 1);
    }
  };

  const handleKeyDown = (e: KeyboardEvent<HTMLFormElement>) => {
    if (e.key === 'Enter') {
      e.preventDefault();
    }
  };

  const onSubmit = async (values: FormValues) => {
    setError('');

    startTransition(async () => {
      const formData = new FormData();
      formData.append('parodontite', values.parodontite);
      formData.append('anni', values.anni);
      formData.append('dentistaVisitaItalia', values.dentistaVisitaItalia);
      formData.append('panaromicaDentale', values.panaromicaDentale);
      if (values.panoramica) {
        formData.append('panoramica', values.panoramica);
      }
      formData.append('nome', values.nome);
      formData.append('email', values.email);
      formData.append('telefono', values.telefono);
      if (values.tuaDomanda) {
        formData.append('tuaDomanda', values.tuaDomanda);
      }
      formData.append('contattato', values.contattato.join(','));

      try {
        await dentalFormSubmit(formData);
        trackEvent('Form_Submit', {
          form_name: 'dental_questionnaire',
          age_range: values.anni,
          name: values.nome,
          email: values.email,
          phone_number: values.telefono,
        });
        form.reset();
      } catch (error) {
        trackEvent('Form_Error', {
          form_name: 'dental_questionnaire',
          error_message: error instanceof Error ? error.message : 'Unknown error',
          step: step,
          age_range: values.anni,
          name: values.nome,
          email: values.email,
          phone_number: values.telefono,
        });
        if (error instanceof Error) {
          setError(error.message);
          return;
        }
        setError('Unknown error happened!');
      }
    });
  };

  const totalSteps = Object.keys(stepsData).length;
  const progressPercentage = Math.round((step / totalSteps) * 100) - 10;

  return (
    <Form {...form}>
      {step === 1 && <FirstStep setState={setStep} />}
      {step >= 2 && (
        <form className='flex h-full w-full items-center justify-center' onKeyDown={handleKeyDown}>
          {step === 2 && (
            <EnumChoosingField
              control={form.control}
              name='parodontite'
              label='Soffre di Parodontite? *'
              options={paradontiteType}
              nextStep={nextStep}
            />
          )}
          {step === 3 && (
            <EnumChoosingField
              control={form.control}
              name='anni'
              label='Quanti anni hai? *'
              options={anniType}
              nextStep={nextStep}
            />
          )}
          {step === 4 && (
            <EnumChoosingField
              control={form.control}
              name='dentistaVisitaItalia'
              label='Hai fatto una visita da un dentista in Italia? *'
              options={siNoType}
              nextStep={nextStep}
            />
          )}
          {step === 5 && (
            <EnumChoosingField
              control={form.control}
              name='panaromicaDentale'
              label='Hai una Panoramica Dentale? *'
              options={siNoType}
              nextStep={nextStep}
            />
          )}
          {step === 6 && (
            <div className='w-10/12 md:w-1/3'>
              <ImageInputField
                control={form.control}
                name='panoramica'
                label='Carica la Panoramica'
                nextStep={nextStep}
              />
            </div>
          )}
          {step === 7 && (
            <div className='w-10/12 md:w-1/3'>
              <FormField
                control={form.control}
                name='nome'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel className='text-2xl font-bold'>Come ti chiami? *</FormLabel>
                    <FormControl>
                      <Input
                        {...field}
                        type='text'
                        placeholder='Inserisci il tuo Nome'
                        className='flex h-[50px] w-full cursor-pointer items-center justify-between rounded-[5px] border-2 border-teal-600 pr-14 text-left font-semibold focus:rounded-[5px] focus:border-teal-600 focus:outline-none focus-visible:ring-transparent'
                      />
                    </FormControl>
                    <FormMessage />
                    {field.value ? (
                      <Button
                        disabled={!!form.formState.errors.nome || field.value.length < 2}
                        onClick={async (e) => {
                          e.preventDefault();
                          await nextStep();
                        }}
                      >
                        OK
                      </Button>
                    ) : null}
                  </FormItem>
                )}
              />
            </div>
          )}
          {step === 8 && (
            <div className='w-10/12 md:w-1/3'>
              <FormField
                control={form.control}
                name='email'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel className='text-2xl font-bold'>Email *</FormLabel>
                    <FormDescription>
                      Si prega di fornire il proprio indirizzo email
                    </FormDescription>
                    <FormControl>
                      <Input
                        {...field}
                        type='email'
                        placeholder='Inserisci la tua email'
                        className='flex h-[50px] w-full cursor-pointer items-center justify-between rounded-[5px] border-2 border-teal-600 pr-14 text-left font-semibold focus:rounded-[5px] focus:border-teal-600 focus:outline-none focus-visible:ring-transparent'
                      />
                    </FormControl>
                    <FormMessage />
                    {field.value ? (
                      <Button
                        disabled={
                          !!form.formState.errors.email ||
                          field.value.length < 2 ||
                          !field.value.includes('@')
                        }
                        onClick={async (e) => {
                          e.preventDefault();
                          await nextStep();
                        }}
                      >
                        OK
                      </Button>
                    ) : null}
                  </FormItem>
                )}
              />
            </div>
          )}
          {step === 9 && (
            <div className='w-10/12 md:w-1/3'>
              <FormField
                control={form.control}
                name='telefono'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel className='text-2xl font-bold'>Telefono *</FormLabel>
                    <FormControl>
                      <PhoneInput
                        {...field}
                        country='it'
                        preferredCountries={['it', 'al']}
                        onChange={(value) => {
                          field.onChange(value);
                        }}
                        inputStyle={{
                          height: '50px',
                          width: '100%',
                          paddingLeft: '3.5rem',
                          paddingRight: '1rem',
                          border: `2px solid ${theme === 'dark' ? '#4FD1C5' : '#319795'}`,
                          borderRadius: '5px',
                          color: theme === 'dark' ? '#CBD5E0' : '#4A5568',
                          fontWeight: '500',
                          outline: 'none',
                          backgroundColor: theme === 'dark' ? '#1A202C' : 'white',
                          transition: 'all 0.3s ease',
                        }}
                        buttonStyle={{
                          borderRight: `2px solid ${theme === 'dark' ? '#4FD1C5' : '#319795'}`,
                          backgroundColor: theme === 'dark' ? '#2D3748' : '#E6FFFA',
                          paddingRight: '0.5rem',
                          borderRadius: '5px 0 0 5px',
                          cursor: 'pointer',
                          background: theme === 'dark' ? '#2D3748' : '#E6FFFA',
                        }}
                        dropdownStyle={{
                          backgroundColor: theme === 'dark' ? '#2D3748' : 'white',
                          border: `2px solid ${theme === 'dark' ? '#4FD1C5' : '#319795'}`,
                          borderRadius: '5px',
                          boxShadow: '0px 4px 12px rgba(0, 0, 0, 0.15)',
                          maxHeight: '240px',
                          overflowY: 'auto',
                        }}
                      />
                    </FormControl>
                    <FormMessage />
                    {field.value ? (
                      <Button
                        disabled={!!form.formState.errors.telefono || field.value.length < 10}
                        onClick={async (e) => {
                          e.preventDefault();
                          await nextStep();
                        }}
                      >
                        OK
                      </Button>
                    ) : null}
                  </FormItem>
                )}
              />
            </div>
          )}
          {step === 10 && (
            <div className='w-10/12 md:w-1/3'>
              <FormField
                control={form.control}
                name='tuaDomanda'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel className='text-2xl font-bold'>La tua domanda</FormLabel>
                    <FormDescription>
                      Fateci sapere i dettagli sulla vostra richiesta
                    </FormDescription>
                    <FormControl>
                      <Textarea
                        {...field}
                        disabled={isPending}
                        placeholder='Fateci sapere i dettagli sulla vostra richiesta'
                      />
                    </FormControl>
                    <FormMessage />
                    <Button
                      disabled={!!form.formState.errors.tuaDomanda}
                      onClick={async (e) => {
                        e.preventDefault();
                        await nextStep();
                      }}
                    >
                      OK
                    </Button>
                  </FormItem>
                )}
              />
            </div>
          )}
          {step === 11 && (
            <div className='w-10/12 md:w-1/3'>
              <CheckboxField
                control={form.control}
                name='contattato'
                label='Contattato *'
                options={contattatoType}
                isPending={isPending}
              />
              {error ? <FormError message={error} /> : null}
              {form.formState.isValid ? (
                <div className='mt-3'>
                  <Button disabled={isPending} onClick={form.handleSubmit(onSubmit)}>
                    Invia la richiesta
                  </Button>
                </div>
              ) : null}
            </div>
          )}
        </form>
      )}
      <div className='fixed z-20 size-12 md:bottom-4 md:right-4 lg:bottom-4 lg:right-4'>
        <div className='fixed bottom-4 right-4 z-20 mr-12 flex items-center space-x-2 rounded-full bg-background p-2 shadow-lg'>
          <Button
            variant='ghost'
            size='icon'
            className='h-8 w-8 rounded-full hover:bg-primary/10'
            aria-label='Decrease progress'
            disabled={isPending}
            onClick={prevStep}
          >
            <ChevronLeft className='h-4 w-4' />
          </Button>
          <div className='relative flex h-2 w-48 items-center justify-center rounded-full bg-secondary'>
            <Progress value={progressPercentage} max={100} className='h-full rounded-full' />
            <span className='absolute text-center text-xs font-medium'>{progressPercentage} %</span>
          </div>
          <Button
            variant='ghost'
            size='icon'
            className='h-8 w-8 rounded-full hover:bg-primary/10'
            aria-label='Increase progress'
            disabled={isPending}
            onClick={nextStep}
          >
            <ChevronRight className='h-4 w-4' />
          </Button>
        </div>
      </div>
    </Form>
  );
};

export default UserForm;
